@@ -93,6 +93,9 @@ yelp_io_channel_new_file (gchar    *file,
 #else
     if (!channel->gzin) {
 #endif
+	yelp_io_free (iochannel);
+	channel = (YelpIOChannel *) g_io_channel_new_file (file, "r", error);
+
 	if (error)
 	    g_set_error (error, YELP_ERROR, YELP_ERROR_IO,
 			 _("The file ‘%s’ could not be read and decoded. "
@@ -100,6 +103,7 @@ yelp_io_channel_new_file (gchar    *file,
 			   "format."),
 			 file);
 	return NULL;
+
     }
 
     iochannel->is_readable  = TRUE;
