@@ -40,6 +40,7 @@
 #include "yelp-man-pager.h"
 #include "yelp-man-parser.h"
 #include "yelp-settings.h"
+#include "yelp-debug.h"
 
 #define STYLESHEET_PATH DATADIR"/yelp/xslt/"
 #define MAN_STYLESHEET  STYLESHEET_PATH"man2html.xsl"
@@ -82,6 +83,65 @@ static YelpManSect *  man_section_process    (xmlNodePtr        node,
 
 static YelpPagerClass *parent_class;
 
+typedef struct _YelpLangEncodings YelpLangEncodings;
+
+struct _YelpLangEncodings {
+    gchar *language;
+    gchar *encoding;
+};
+
+/* http://www.w3.org/International/O-charset-lang.html */
+static const YelpLangEncodings langmap[] = {
+    { "C",     "ISO-8859-1" },
+    { "af",    "ISO-8859-1" },
+    { "ar",    "ISO-8859-6" },
+    { "bg",    "ISO-8859-5" },
+    { "be",    "ISO-8859-5" },
+    { "ca",    "ISO-8859-1" },
+    { "cs",    "ISO-8859-2" },
+    { "da",    "ISO-8859-1" },
+    { "de",    "ISO-8859-1" },
+    { "el",    "ISO-8859-7" },
+    { "en",    "ISO-8859-1" },
+    { "eo",    "ISO-8859-3" },
+    { "es",    "ISO-8859-1" },
+    { "et",    "ISO-8859-15" },
+    { "eu",    "ISO-8859-1" },
+    { "fi",    "ISO-8859-1" },
+    { "fo",    "ISO-8859-1" },
+    { "fr",    "ISO-8859-1" },
+    { "ga",    "ISO-8859-1" },
+    { "gd",    "ISO-8859-1" },
+    { "gl",    "ISO-8859-1" },
+    { "hu",    "ISO-8859-2" },
+    { "id",    "ISO-8859-1" }, /* is this right */
+    { "mt",    "ISO-8859-3" },
+    { "is",    "ISO-8859-1" },
+    { "it",    "ISO-8859-1" },
+    { "iw",    "ISO-8859-8" },
+    { "ja",    "EUC-JP" },
+    { "ko",    "EUC-KR" },
+    { "lt",    "ISO-8859-13" },
+    { "lv",    "ISO-8859-13" },
+    { "mk",    "ISO-8859-5" },
+    { "mt",    "ISO-8859-3" },
+    { "no",    "ISO-8859-1" },
+    { "pl",    "ISO-8859-2" },
+    { "pt_BR", "ISO-8859-1" },
+    { "ro",    "ISO-8859-2" },
+    { "ru",    "KOI8-R" },
+    { "sl",    "ISO-8859-2" },
+    { "sr",    "ISO-8859-2" }, /* Latin, not cyrillic */
+    { "sk",    "ISO-8859-2" },
+    { "sv",    "ISO-8859-1" },
+    { "tr",    "ISO-8859-9" },
+    { "uk",    "ISO-8859-5" },
+    { "zh_CN", "BIG5" },
+    { "zh_TW", "BIG5" },
+    { NULL,    NULL },
+};
+	
+
 GType
 yelp_man_pager_get_type (void)
 {
@@ -161,9 +221,12 @@ man_pager_parse (YelpPager *pager)
 {
     YelpDocInfo   *doc_info;
     gchar         *filename;
+    const gchar   *language;
+    const gchar   *encoding;
     YelpManParser *parser;
     xmlDocPtr      doc;
-    GError        *error;
+    GError        *error = NULL;
+    gint           i;
 
     g_return_val_if_fail (YELP_IS_MAN_PAGER (pager), FALSE);
 
@@ -172,8 +235,27 @@ man_pager_parse (YelpPager *pager)
 
     g_object_ref (pager);
 
+    /* We use the language to determine which encoding the manual
+     * page is in */ 
+    language = yelp_doc_info_get_language (doc_info);
+    debug_print (DB_INFO, "The language of the man page is %s\n", language);
+
+    /* default encoding if the language doesn't match below */
+    encoding = g_getenv("MAN_ENCODING");
+    if (encoding == NULL)
+	encoding = "ISO-8859-1";
+
+    if (language != NULL) {
+	for (i=0; langmap[i].language != NULL; i++) {
+	    if (g_str_equal (language, langmap[i].language)) {
+		encoding = langmap[i].encoding;
+		break;
+	    }
+	}
+    }
+    
     parser = yelp_man_parser_new ();
-    doc = yelp_man_parser_parse_file (parser, filename);
+    doc = yelp_man_parser_parse_file (parser, filename, encoding);
     yelp_man_parser_free (parser);
 
     if (doc == NULL) {
